def cluster_list(self, path):
    return [u'general/clusters/cluster-google-merchant']

def cluster_data(self, key):
    return {u'ids': [123, 456]}

def feature_list(self, path):
    return [u'general/feature-toggle/google-merchant']

def feature_data(self, key):
    return {
        u'default': False,
        u'clusters': [u'general/clusters/cluster-google-merchant'],
        u'enable': True}

def invalid_feature_data(self, key):
    return {
        u'clusters': [u'general/clusters/cluster-google-merchant'],
        u'enable': True}

def invalid_cluster_data(self, key):
    return {u'ids': ["123", 456]}

def cache_init_data(self, key):
    if 'cluster' in key :
        return cluster_data(self, key)
    else:
        return feature_data(self, key)

def cache_init_list(self, path):
    if path == self.toggle_path:
        return feature_list(self, path)
    else:
        return cluster_list(self, path)

def cache_update_cluster_data(self, key):
    if 'cluster' in key :
        return {u'ids': [124]}
    else:
        return feature_data(self, key)

def cache_update_feature_cluster_data(self, key):
    if 'cluster' in key :
        return {u'ids': [124]}
    else:
       return {u'default': False, u'clusters': [u'general/clusters/cluster-google-merchant'], u'enable': False}


from remote_config.base import RemoteConfig

from jsonschema.exceptions import ValidationError
import mock_data as mock
import pytest


def test_feature_disable():
    RemoteConfig().features = {
        u'google-merchant': {
            u'default': False,
            u'enable': False,
            'stores': set()
        }
    }
    assert RemoteConfig().get_feature('google-merchant', 0) == False

def test_feature_in_cluster():
    RemoteConfig().features = {
        u'google-merchant': {
            u'default': False,
            u'enable': True,
            'stores': set([456, 123])
        }
    }
    assert RemoteConfig().get_feature('google-merchant', 123) == True

def test_feature_default_false():
    RemoteConfig().features = {
        u'google-merchant': {
            u'default': False,
            u'enable': True,
            'stores': set([456])
        }
    }
    assert RemoteConfig().get_feature('google-merchant', 123) == False

def test_feature_default_true():
    RemoteConfig().features = {
        u'google-merchant': {
            u'default': True,
            u'enable': True,
            u'clusters': [u'general/clusters/cluster-google-merchant'],
            'stores': set([456])
        }
    }
    assert RemoteConfig().get_feature('google-merchant', 123) == True

def test_feature_not_exist():
    assert RemoteConfig().get_feature('feature-not-exist', 123) == False

def test_invalid_cluster_consumer():
    RemoteConfig().clusters = None
    RemoteConfig._list_folders = mock.cluster_list
    RemoteConfig._get_value = mock.invalid_cluster_data
    with pytest.raises(ValidationError):
        RemoteConfig()._load_clusters()

def test_invalid_feature_consumer():
    RemoteConfig().clusters = None
    RemoteConfig._list_folders = mock.cluster_list
    RemoteConfig._get_value = mock.cluster_data
    RemoteConfig()._load_clusters()
    RemoteConfig._list_folders = mock.feature_list
    RemoteConfig._get_value = mock.invalid_feature_data
    with pytest.raises(ValidationError):
        RemoteConfig()._load_features()

def test_cluster_feature_consumer():
    rc = RemoteConfig()
    rc.clusters = None
    RemoteConfig._list_folders = mock.cluster_list
    RemoteConfig._get_value = mock.cluster_data
    rc._load_clusters()
    RemoteConfig._list_folders = mock.feature_list
    RemoteConfig._get_value = mock.feature_data
    rc._load_features()
    assert rc.get_feature('google-merchant', 123) == True

def test_cluster_feature_update_cache():
    rc = RemoteConfig()
    RemoteConfig._list_folders = mock.cache_init_list
    RemoteConfig._get_value = mock.cache_init_data
    rc._load_clusters()
    assert rc.get_feature('google-merchant', 124) == False
    RemoteConfig._list_folders = mock.cache_init_list
    RemoteConfig._get_value = mock.cache_update_cluster_data
    rc._load_clusters()
    assert rc.get_feature('google-merchant', 124) == True

def test_cluster_feature_cluster_update_cache():
    rc = RemoteConfig()
    RemoteConfig._list_folders = mock.cache_init_list
    RemoteConfig._get_value = mock.cache_init_data
    rc._load_clusters()
    rc._load_features()
    assert rc.get_feature('google-merchant', 123) == True
    RemoteConfig._list_folders = mock.cache_init_list
    RemoteConfig._get_value = mock.cache_update_cluster_data
    rc._load_clusters()
    rc._load_features()
    assert rc.get_feature('google-merchant', 123) == False


import os
import re

from setuptools import setup, find_packages


PACKAGE = "remote_config"
NAME = "remote-config"
DESCRIPTION = "Remote config and feature toggle based on Consul."
AUTHOR = "Loja Integrada"
AUTHOR_EMAIL = "suporte@lojaintegrada.com.br"
URL = "https://github.com/lojaintegrada/remote-config"
METADATA = dict(
    re.findall("__([a-z]+)__ = \"([^']+)\"", open("{}/__init__.py".format(PACKAGE)).read()))
VERSION = METADATA['version']

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license="MIT",
    url=URL,
    packages=find_packages(exclude=['tests']),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet",
    ],
    zip_safe=False,
    tests_require=['pytest','pytest-cov'],
    install_requires=[
        "python-consul==1.1.0",
        "APScheduler==3.7.0",
        "jsonschema==3.2.0"
    ])



from jsonschema import validate


feature_schema = {
    "type" : "object",
    "properties" : {
        "enable" : {"type" : "boolean"},
        "default" : {"type" : "boolean"},
        "clusters" : {
            "type" : "array", 
            "items": {
            "type": "string"
            }
        },
     },
     "required": ["enable", "default", "clusters"]
}

cluster_schema = {
    "type" : "object",
    "properties" : {
        "ids" : {
            "type" : "array", 
            "items": {
            "type": "number"
            }
        },
     },
    "required": ["ids"],
}

def validate_feature(data):
    validate(data, feature_schema)

def validate_cluster(data):
    validate(data, cluster_schema)
